#!/usr/bin/perl -I/home/phil/perl/cpan/DataTableText/lib -I/home/phil/perl/makeWithPerl/ -I/home/phil/perl/cpan/JavaDoc/lib -I/home/phil/perl/cpan/MakeWithPerl/lib/
#-------------------------------------------------------------------------------
# Test MakeWithPerl
# Philip R Brenan at gmail dot com, Appa Apps Ltd, 2021
#-------------------------------------------------------------------------------

use MakeWithPerl;

MakeWithPerl::test();
